/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmSvrHelpers.h
 * version 4
 */
#ifndef TSM_SVRHELPERS_H
#define TSM_SVRHELPERS_H

#include <map>
#include <set>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "TsmThreadType.h"
#include "TsmThread.h"
#include "TsmThreadState.h"
#include "Tsm.h"
using namespace std;

namespace TsmRTE
{
  class TsmThreadData
  {
  public:
    TsmThreadType  Type;
    int            ThreadId;
    TsmThread*     pTsmThread;
    TsmThreadState State;

    bool hasMessage()
    {
      return pTsmThread->hasMessage();
    }

    string toState()
    {
      stringstream ss;

      ss << "Thread " << this->ThreadId;
      ss << " Type " << this->Type;
      ss << " State " << this->State;
      ss << " BoostId " << this->pTsmThread->getThread()->get_id();
      return ss.str();
    }
  };

  class TsmSnapshot
  {
  public:
    TsmSnapshot*  Parent;
    string        SnapClasses;
    string        SnapCbThread;
    string        SnapThreads;

  public:
    TsmSnapshot( string sSnapClasses, string sSnapCbThread, string sSnapThreads )
      : SnapClasses( sSnapClasses )
      , SnapCbThread( sSnapCbThread )
      , SnapThreads( sSnapThreads )
    {
    }

    string toString()
    {
      return "{" + SnapClasses + " &" + SnapCbThread + " &" + SnapThreads + "}";
    }

    bool equals( void* o2 )
    {
      TsmSnapshot* ss2 = (TsmSnapshot*) o2;
      return ( (this->SnapClasses.compare(ss2->SnapClasses)==0)
            && (this->SnapCbThread.compare(ss2->SnapCbThread)==0)
            && (this->SnapThreads.compare(ss2->SnapThreads)==0) );
    }
  };

  enum TsmScenItemKind
  {
    RELEASE_CB_MSG,
    RELEASE_CB_RV2,
    RELEASE_MSG,
    RELEASE_RV2,
    NATIVE,
    CHOICE,
    QO
  };

  class TsmScenItem
  {
  public:
    TsmScenItemKind  Kind;
    int              Index;
    TsmClass*        Receiver;
    TsmThreadData*   ThreadData;

    string toString()
    {
      stringstream ss;

      switch ( Kind )
      {
      case RELEASE_CB_MSG :
        return "CB_MSG";
      case RELEASE_CB_RV2 :
        return "CB_RV2";
      case RELEASE_MSG :
        ss << "MSG ";
        ss << ThreadData->ThreadId;
        return ss.str();
      case RELEASE_RV2 :
        ss << "RV2 ";
        ss << ThreadData->ThreadId;
        return ss.str();
      case NATIVE :
        ss << "NATIVE";
        ss.width(5);
        ss << Index;
        ss << " " + Receiver->FullName();
        return ss.str();
      case CHOICE :
        ss << "CHOICE ";
        ss << Index;
        return ss.str();
      case QO :
        return "QO";
      }
      return "TismBUG";
    }
  };

  typedef vector<boost::shared_ptr<TsmScenItem> > TsmVectorSpSi;
  class TsmScenario
  {
  public:
    static int            ScenarioId;
    int                   Id;
    TsmScenario*          Parent;
    TsmVectorSpSi*        Items;
    TsmSnapshot*          Snapshot;

  private:
    void NewScenario( TsmScenario* oParent )
    {
      Id = ScenarioId++;
      Parent = oParent;
      Items = new TsmVectorSpSi;
      Snapshot = NULL;
    }

  public:
    TsmScenario()
    {
      ScenarioId = 0;
      NewScenario( NULL );
    }

    TsmScenario( TsmScenario* oParent, TsmVectorSpSi* oItems )
    {
      Id = ScenarioId++;
      Parent = oParent;
      Items = new TsmVectorSpSi( *oItems );
      Snapshot = oParent->Snapshot;
    }

    ~TsmScenario()
    {
      delete Items;
    }

    int getParentId()
    {
      return ( Parent == NULL ) ? -1 : Parent->Id;
    }

    TsmScenario* Extend( TsmScenItem* oTsmScenItem )
    {
      TsmScenario* lTsmScenario = new TsmScenario( this, this->Items );
      lTsmScenario->Add( oTsmScenItem );

      return lTsmScenario;
    }

    TsmScenario* ExtendChoice( TsmScenItem* oTsmScenItem )
    {
      TsmScenario* lTsmScenario = new TsmScenario( this->Parent, this->Items );
      lTsmScenario->Add( oTsmScenItem );

      return lTsmScenario;
    }

    void Add( TsmScenItem* oTsmScenItem )
    {
      boost::shared_ptr<TsmScenItem> SpSi(oTsmScenItem);
      Items->push_back( SpSi );
    }

    void Update( TsmSnapshot* oSnapshot )
    {
      oSnapshot->Parent = this->Snapshot;
      this->Snapshot = oSnapshot;
    }

    bool HasCycle()
    {
      TsmSnapshot* lSnapshot = this->Snapshot;

      while ( NULL != (lSnapshot = lSnapshot->Parent) )
      {
        if ( lSnapshot->equals( this->Snapshot ) )
        {
          return true;
        }
      }
      return false;
    }

    string toString()
    {
      stringstream sb;
      for ( unsigned int i=0; i<Items->size(); i++ )
      {
        sb.width(3);
        sb << i << " ";
        sb << Items->at(i)->toString() << endl;
      }
      return sb.str();
    }
  };

  class TsmSnapdata
  {
  public:
    int          NrRecords;
    int          NrCalls;
    TsmScenario* Scenario;

    TsmSnapdata( int iNrRecords, int iNrCalls, TsmScenario* oScenario )
      : NrRecords( iNrRecords )
      , NrCalls( iNrCalls )
      , Scenario( oScenario )
    {
    }
  };

  /*
   * Received client natives have negative values.
   */
  class TsmNativeLists
  {
  public:
    multiset<int>  ClientNatives;
    multiset<int>  ServerNatives;

    TsmNativeLists()
    {
    }

    ~TsmNativeLists()
    {
    }
  };
}
#endif
